/********************************** (C) COPYRIGHT  *******************************
 * File Name          : iap.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2020/12/16
 * Description        : IAP
 *  Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/
#include "iap.h"
#include "string.h"
#include "core_riscv.h"

/******************************************************************************/
#define USBD_DATA_SIZE               64
u8 EP2_Rx_Buffer[USBD_DATA_SIZE];
#define  isp_cmd_t   ((isp_cmd  *)EP2_Rx_Buffer)

u16 Bin_Len;  //用户代码总长度

/*********************************************************************
 * @fn      CAN_Mode_Init
 *
 * @brief   Initializes CAN communication test mode.
 *          Bps =Fpclk1/((tpb1+tbs2+1)*brp)
 *
 * @param   tsjw - CAN synchronisation jump width.
 *          tbs2 - CAN time quantum in bit segment 1.
 *          tbs1 - CAN time quantum in bit segment 2.
 *          brp - Specifies the length of a time quantum.
 *          mode - Test mode.
 *            CAN_Mode_Normal.
 *            CAN_Mode_LoopBack.
 *            CAN_Mode_Silent.
 *            CAN_Mode_Silent_LoopBack.
 *
 * @return  none
 */
void CAN_Mode_Init(u8 tsjw, u8 tbs2, u8 tbs1, u16 brp, u8 mode)
{
    GPIO_InitTypeDef      GPIO_InitSturcture = {0};
    CAN_InitTypeDef       CAN_InitSturcture = {0};
    CAN_FilterInitTypeDef CAN_FilterInitSturcture = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO | RCC_APB2Periph_GPIOB, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN1, ENABLE);

    GPIO_PinRemapConfig(GPIO_Remap1_CAN1, ENABLE);

    GPIO_InitSturcture.GPIO_Pin = GPIO_Pin_9;
    GPIO_InitSturcture.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitSturcture.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitSturcture);

    GPIO_InitSturcture.GPIO_Pin = GPIO_Pin_8;
    GPIO_InitSturcture.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(GPIOB, &GPIO_InitSturcture);

    CAN_InitSturcture.CAN_TTCM = DISABLE;
    CAN_InitSturcture.CAN_ABOM = DISABLE;
    CAN_InitSturcture.CAN_AWUM = DISABLE;
    CAN_InitSturcture.CAN_NART = ENABLE;
    CAN_InitSturcture.CAN_RFLM = DISABLE;
    CAN_InitSturcture.CAN_TXFP = DISABLE;
    CAN_InitSturcture.CAN_Mode = mode;
    CAN_InitSturcture.CAN_SJW = tsjw;
    CAN_InitSturcture.CAN_BS1 = tbs1;
    CAN_InitSturcture.CAN_BS2 = tbs2;
    CAN_InitSturcture.CAN_Prescaler = brp;
    CAN_Init(CAN1, &CAN_InitSturcture);

    CAN_FilterInitSturcture.CAN_FilterNumber = 0;

    /* identifier/mask mode, One 32-bit filter, StdId: 0x317 */
    CAN_FilterInitSturcture.CAN_FilterMode = CAN_FilterMode_IdMask;
    CAN_FilterInitSturcture.CAN_FilterScale = CAN_FilterScale_32bit;
    CAN_FilterInitSturcture.CAN_FilterIdHigh = 0x62E0;
    CAN_FilterInitSturcture.CAN_FilterIdLow = 0;
    CAN_FilterInitSturcture.CAN_FilterMaskIdHigh = 0xFFE0;
    CAN_FilterInitSturcture.CAN_FilterMaskIdLow = 0x0006;

    CAN_FilterInitSturcture.CAN_FilterFIFOAssignment = CAN_Filter_FIFO0;
    CAN_FilterInitSturcture.CAN_FilterActivation = ENABLE;
    CAN_FilterInit(&CAN_FilterInitSturcture);
}

/*********************************************************************
 * @fn      CAN_Send_Msg
 *
 * @brief   CAN Transmit function.
 *
 * @param   msg - Transmit data buffer.
 *          len - Data length(<=8).
 *
 * @return  0 - Send successful.
 *          1 - Send failed.
 */
u8 CAN_Send_Msg(u8 *msg, u8 len)
{
    u8  mbox;
    u16 i = 0;

    CanTxMsg CanTxStructure;

    CanTxStructure.StdId = CAN_ID;
    CanTxStructure.IDE = CAN_Id_Standard;
    CanTxStructure.RTR = CAN_RTR_Data;
    CanTxStructure.DLC = len;

    for(i = 0; i < len; i++) {
        CanTxStructure.Data[i] = msg[i];
    }

    mbox = CAN_Transmit(CAN1, &CanTxStructure);
    i = 0;

    while((CAN_TransmitStatus(CAN1, mbox) != CAN_TxStatus_Ok) && (i < 0xFFF))
    {
        i++;
    }

    if(i == 0xFFF)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

/*********************************************************************
 * @fn      CAN_Receive_Msg
 *
 * @brief   CAN Receive function.
 *
 * @param   buf - Receive data buffer.
 *          len - Data length(<=8).
 *
 * @return  CanRxStructure.DLC - Receive data length.
 */
u8 CAN_Receive_Msg(u8 *buf)
{
    u8 i;

    CanRxMsg CanRxStructure;

    while(((uint8_t)(CAN1->RFIFO0&(uint32_t)0x03)) == 0)
    {
    }
    CAN_Receive(CAN1, CAN_FIFO0, &CanRxStructure);

    for(i = 0; i < 8; i++) {
        buf[i] = CanRxStructure.Data[i];
    }

    return CanRxStructure.DLC;
}

/*********************************************************************
 * @fn      CRC_Verify_Init
 *
 * @brief
 *
 * @return
 */
void CRC_Verify_Init(void)
{
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_CRC, ENABLE);
    CRC_ResetDR();
}

/*********************************************************************
 * @fn      RecData_Deal
 *
 * @brief   UART-USB deal date
 *
 * @return  ERR_ERROR - ERROR
 *          ERR_SCUESS - SCUESS
 *          ERR_End - End
 */

u8 RecData_Deal(void)
{
     u8 i, s, Lenth;
     u8 CAN_Rx[2];

     Lenth = isp_cmd_t->Len;
     switch ( isp_cmd_t->Cmd) {
         case CMD_IAP_START:{
             CRC_Verify_Init();
             Bin_Len = 0;
             CAN_Send_Msg(EP2_Rx_Buffer, 4);
             CAN_Receive_Msg(CAN_Rx);
             s = ERR_SCUESS;


             break;
         }
         case CMD_IAP_PROM:{
             //不满60字节，补0xFF
             if(Lenth != 60){
                 for(i=0; i<(60 - Lenth%60); i++){
                     isp_cmd_t->data[Lenth + i] = 0xFF;
                 }

                 Lenth = 64;
             }
             for(i=0; i<8; i++){
                 CAN_Send_Msg(&EP2_Rx_Buffer[8*i], 8);
             }
             CAN_Receive_Msg(CAN_Rx);

             s = ERR_SCUESS;
             if((CAN_Rx[0]!=0) || (CAN_Rx[1]!=0)){
                 s = ERR_ERROR;
                 break;
             }
             break;
         }
         case CMD_IAP_VERIFY:{
             /* 4Byte校验 */
             //不满60字节，补0xFF
             if(Lenth != 60){
                 for(i=0; i<(60 - Lenth%60); i++){
                     isp_cmd_t->data[Lenth + i] = 0xFF;
                 }

                 Lenth = 64;
             }

             for(i=0; i<8; i++){
                 CAN_Send_Msg(&EP2_Rx_Buffer[8*i], 8);
                 Delay_Ms(10);
                 //硬件CRC校验
                 Bin_Len += 8;
             }

             Bin_Len -= 4; /* 去掉包头 */

             CAN_Receive_Msg(CAN_Rx);

             s = ERR_SCUESS;

             if((CAN_Rx[0]!=0) || (CAN_Rx[1]!=0)){
                 s = ERR_ERROR;
                 break;
             }

             /* 完整bin文件硬件CRC校验 */
             for(i=0; i<15; i++){
                 CRC->DATAR = *(u32*)&isp_cmd_t->data[4*i];
             }

             break;
         }

         case CMD_IAP_END:{
             /* 完整bin文件，硬件CRC校验 */
             isp_cmd_t->Cmd = CMD_IAP_ALL_VERIFY;
             isp_cmd_t->Len = 0x04;
             *(u16*)&isp_cmd_t->Rev[0] = Bin_Len;
             *(u32*)&isp_cmd_t->data[0] = CRC->DATAR;
             CAN_Send_Msg(EP2_Rx_Buffer, 8);

             CAN_Receive_Msg(CAN_Rx);

             if((CAN_Rx[0]!=0) || (CAN_Rx[1]!=0)){
                 PR("All Bin Verify ERR\r\n");
             }
             else{
                 PR("All Bin Verify SUC\r\n");
             }

             /* 从缓存区搬数据到用户区，并校验 */
             isp_cmd_t->Cmd = CMD_IAP_BufToApp;
             isp_cmd_t->Len = 0x02;
             *(u16*)&isp_cmd_t->Rev[0] = 0;
             CAN_Send_Msg(EP2_Rx_Buffer, 4);

             CAN_Receive_Msg(CAN_Rx);

             if((CAN_Rx[0]!=0) || (CAN_Rx[1]!=0)){
                 PR("BufToApp Verify ERR\r\n");
             }
             else{
                 PR("BufToApp Verify SUC\r\n");
             }

             /* 结束命令 */
             isp_cmd_t->Cmd = CMD_IAP_END;
             isp_cmd_t->Len = 0x02;
             *(u16*)&isp_cmd_t->Rev[0] = 0;
             CAN_Send_Msg(EP2_Rx_Buffer, 4);

             s = ERR_End;
             key_flag=0;
             PR("#2\r\n");
             break;
         }
         default:{
             s = ERR_ERROR;
             break;
         }
     }

     return s;
 }


/*********************************************************************
 * @fn      GPIO_Cfg_init
 *
 * @brief   GPIO init
 *
 * @return  none
 */
void GPIO_Cfg_init(void)
{
    GPIO_InitTypeDef  GPIO_InitStructure;

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA,ENABLE);
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
}


/*********************************************************************
 * @fn      KEY_Check
 *
 * @brief   Check PA0 state
 *
 * @return
 */
u8 KEY_Check(void)
{
    u8 i, cnt=0;

    GPIO_Cfg_init();

    for(i=0; i<10; i++){
        if(GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0)==0) cnt++;
        Delay_Ms(50);
    }

    if(cnt>8){
        key_flag=1;
        PR("#1\r\n");
        return 0;
    }
    else return 1;
}
